@@ -59,6 +59,8 @@ module Milter::Manager
       _maintenance_interval = nil if _maintenance_interval.zero?
       result << "manager.maintenance_interval = #{_maintenance_interval.inspect}\n"
       result << "manager.suspend_time_on_unacceptable = #{suspend_time_on_unacceptable.inspect}\n"
+      result << "manager.max_connections = #{max_connections.inspect}\n"
+      result << "manager.max_file_descriptors = #{max_file_descriptors.inspect}\n"
       result << "manager.custom_configuration_directory = #{custom_configuration_directory.inspect}\n"
       result << "\n"
 
@@ -83,6 +85,7 @@ module Milter::Manager
         result << "  milter.enabled = #{egg.enabled?}\n"
         fallback_status = egg.fallback_status.nick
         result << "  milter.fallback_status = #{fallback_status.inspect}\n"
+        result << "  milter.evaluation_mode = #{egg.evaluation_mode?}\n"
         condition_names = egg.applicable_conditions.collect do |condition|
           condition.name
         end
@@ -349,12 +352,17 @@ module Milter::Manager
               if @egg_config.has_key?("fallback_status")
                 milter.fallback_status = @egg_config["fallback_status"]
               end
+              if @egg_config.has_key?("evaluation_mode")
+                milter.evaluation_mode = @egg_config["evaluation_mode"]
+              end
             end
             @egg_config = nil
           when "milter_applicable_condition"
             @egg_config["applicable_conditions"] << text
           when "milter_enabled"
             @egg_config["enabled"] = text == "true"
+          when "milter_evaluation_mode"
+            @egg_config["evaluation_mode"] = text == "true"
           when /\Amilter_/
             @egg_config[$POSTMATCH] = text
           else
@@ -410,7 +418,7 @@ module Milter::Manager
             available_locals = ["name", "description",
                                 "enabled", "connection_spec",
                                 "command", "command_options",
-                                "fallback_status"]
+                                "fallback_status", "evaluation_mode"]
             case local
             when "applicable_conditions"
               @egg_config["applicable_conditions"] = []
@@ -642,6 +650,24 @@ module Milter::Manager
         @configuration.suspend_time_on_unacceptable
       end
 
+      def max_connections=(n_connections)
+        n_connections ||= Milter::Client::DEFAULT_MAX_CONNECTIONS
+        @configuration.max_connections = n_connections
+      end
+
+      def max_connections
+        @configuration.max_connections
+      end
+
+      def max_file_descriptors=(n_descriptors)
+        n_descriptors ||= 0
+        @configuration.max_file_descriptors = n_descriptors
+      end
+
+      def max_descriptors
+        @configuration.max_descriptors
+      end
+
       def custom_configuration_directory=(directory)
         @configuration.custom_configuration_directory = directory
       end
