@@ -15,8 +15,6 @@
 
 require 'milter/manager/init-detector'
 require 'milter/manager/enma-socket-detector'
-require 'milter/manager/clamav-milter-socket-detector'
-require 'milter/manager/milter-greylist-socket-detector'
 
 module Milter::Manager
   class RedHatInitDetector
@@ -27,7 +25,8 @@ module Milter::Manager
     end
 
     def enabled?
-      !guess_spec.nil? and !rc_files.empty?
+      !guess_spec.nil? and !rc_files.empty? and
+        !(clamav_milter_0_95_or_later? and clamav_milter_example?)
     end
 
     def detect_enma_connection_spec
@@ -40,8 +39,7 @@ module Milter::Manager
     end
 
     def detect_clamav_milter_connection_spec
-      conf = clamav_milter_conf || etc_file("clamav-milter.conf")
-      Milter::Manager::ClamAVMilterSocketDetector.new(conf).detect
+      clamav_milter_config_parser.milter_socket
     end
 
     def clamav_milter?
@@ -53,8 +51,7 @@ module Milter::Manager
     end
 
     def detect_milter_greylist_connection_spec
-      conf = milter_greylist_conf || etc_file("mail", "greylist.conf")
-      Milter::Manager::MilterGreylistSocketDetector.new(conf).detect
+      milter_greylist_config_parser.socket
     end
 
     def milter_greylist?
@@ -121,12 +118,16 @@ module Milter::Manager
     end
 
     def clamav_milter_conf
-      extract_parameter_from_flags(@variables["CLAMAV_FLAGS"], "--config-file")
+      flags = @variables["CLAMAV_FLAGS"]
+      extract_parameter_from_flags(flags, "--config-file") ||
+        extract_parameter_from_flags(flags, "-c") ||
+        etc_file("clamav-milter.conf")
     end
 
     def milter_greylist_conf
       extract_parameter_from_flags(@variables["OPTIONS"], "-f") ||
-        @info["config"]
+        @info["config"] ||
+        etc_file("mail", "greylist.conf")
     end
 
     def guess_spec
