@@ -1,4 +1,4 @@
-# Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -144,6 +144,32 @@ class TestConfigurationLoader < Test::Unit::TestCase
     assert_equal(10, @configuration.suspend_time_on_unacceptable)
   end
 
+  def test_manager_max_connections
+    assert_equal(0, @configuration.max_connections)
+
+    @loader.manager.max_connections = 5
+    assert_equal(5, @configuration.max_connections)
+
+    @loader.manager.max_connections = nil
+    assert_equal(0, @configuration.max_connections)
+
+    @loader.manager.max_connections = 10
+    assert_equal(10, @configuration.max_connections)
+  end
+
+  def test_manager_max_file_descriptors
+    assert_equal(0, @configuration.max_file_descriptors)
+
+    @loader.manager.max_file_descriptors = 5
+    assert_equal(5, @configuration.max_file_descriptors)
+
+    @loader.manager.max_file_descriptors = nil
+    assert_equal(0, @configuration.max_file_descriptors)
+
+    @loader.manager.max_file_descriptors = 10
+    assert_equal(10, @configuration.max_file_descriptors)
+  end
+
   def test_manager_configuration_directory
     assert_nil(@configuration.custom_configuration_directory)
     @loader.manager.custom_configuration_directory = "/tmp/milter-manager/"
@@ -197,6 +223,7 @@ EOX
       <name>child-milter</name>
       <enabled>true</enabled>
       <fallback-status>accept</fallback-status>
+      <evaluation-mode>false</evaluation-mode>
       <connection-spec>unix:/tmp/socket</connection-spec>
       <applicable-conditions>
         <applicable-condition>remote-network</applicable-condition>
