@@ -45,6 +45,18 @@ class TestConfiguration < Test::Unit::TestCase
     assert_equal(10, @configuration.suspend_time_on_unacceptable)
   end
 
+  def test_max_connections
+    assert_equal(0, @configuration.max_connections)
+    @configuration.max_connections = 10
+    assert_equal(10, @configuration.max_connections)
+  end
+
+  def test_max_file_descriptors
+    assert_equal(0, @configuration.max_file_descriptors)
+    @configuration.max_file_descriptors = 10
+    assert_equal(10, @configuration.max_file_descriptors)
+  end
+
   def test_package
     @configuration.package_platform = "pkgsrc"
     assert_equal("pkgsrc", @configuration.package_platform)
@@ -141,6 +153,7 @@ class TestConfiguration < Test::Unit::TestCase
                   "      <name>#{name}</name>",
                   "      <enabled>true</enabled>",
                   "      <fallback-status>accept</fallback-status>",
+                  "      <evaluation-mode>false</evaluation-mode>",
                   "      <connection-spec>#{spec}</connection-spec>",
                   "    </milter>",
                   "  </milters>",
@@ -157,6 +170,7 @@ class TestConfiguration < Test::Unit::TestCase
                   "      <name>#{name}</name>",
                   "      <enabled>true</enabled>",
                   "      <fallback-status>accept</fallback-status>",
+                  "      <evaluation-mode>false</evaluation-mode>",
                   "      <connection-spec>#{spec}</connection-spec>",
                   "    </milter>",
                   "  </milters>",
@@ -200,6 +214,8 @@ manager.daemon = false
 manager.pid_file = nil
 manager.maintenance_interval = 100
 manager.suspend_time_on_unacceptable = 5
+manager.max_connections = 0
+manager.max_file_descriptors = 0
 manager.custom_configuration_directory = nil
 
 controller.connection_spec = nil
@@ -230,6 +246,7 @@ EOD
       milter.description = "The second milter"
       milter.connection_spec = "inet:2929"
       milter.enabled = false
+      milter.evaluation_mode = true
     end
 
     assert_equal(<<-EOD.strip,
@@ -249,6 +266,8 @@ manager.daemon = false
 manager.pid_file = nil
 manager.maintenance_interval = 100
 manager.suspend_time_on_unacceptable = 5
+manager.max_connections = 0
+manager.max_file_descriptors = 0
 manager.custom_configuration_directory = nil
 
 controller.connection_spec = nil
@@ -266,6 +285,7 @@ define_milter("milter1") do |milter|
   milter.description = "The first milter"
   milter.enabled = true
   milter.fallback_status = "accept"
+  milter.evaluation_mode = false
   milter.applicable_conditions = ["S25R"]
   milter.command = nil
   milter.command_options = nil
@@ -281,6 +301,7 @@ define_milter("milter2") do |milter|
   milter.description = "The second milter"
   milter.enabled = false
   milter.fallback_status = "accept"
+  milter.evaluation_mode = true
   milter.applicable_conditions = []
   milter.command = nil
   milter.command_options = nil
