@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -43,15 +43,20 @@ struct _MilterAgentPrivate
     MilterReader *reader;
     MilterWriter *writer;
     gboolean finished;
+    guint tag;
 };
 
 enum
 {
     PROP_0,
     PROP_READER,
-    PROP_DECODER
+    PROP_DECODER,
+    PROP_TAG
 };
 
+static GStaticMutex auto_tag_mutex = G_STATIC_MUTEX_INIT;
+static guint auto_tag = 0;
+
 static void         finished           (MilterFinishedEmittable *emittable);
 
 MILTER_IMPLEMENT_ERROR_EMITTABLE(error_emittable_init);
@@ -106,9 +111,29 @@ milter_agent_class_init (MilterAgentClass *klass)
                                G_PARAM_READABLE);
     g_object_class_install_property(gobject_class, PROP_DECODER, spec);
 
+    spec = g_param_spec_uint("tag",
+                             "Tag",
+                             "The tag of the agent",
+                             0, G_MAXUINT, 0,
+                             G_PARAM_READABLE);
+    g_object_class_install_property(gobject_class, PROP_TAG, spec);
+
     g_type_class_add_private(gobject_class, sizeof(MilterAgentPrivate));
 }
 
+static void
+apply_tag (MilterAgentPrivate *priv)
+{
+    if (priv->encoder)
+        milter_encoder_set_tag(priv->encoder, priv->tag);
+    if (priv->decoder)
+        milter_decoder_set_tag(priv->decoder, priv->tag);
+    if (priv->writer)
+        milter_writer_set_tag(priv->writer, priv->tag);
+    if (priv->reader)
+        milter_reader_set_tag(priv->reader, priv->tag);
+}
+
 static GObject *
 constructor (GType type, guint n_props, GObjectConstructParam *props)
 {
@@ -129,6 +154,14 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
         priv->decoder = agent_class->decoder_new(agent);
     if (agent_class->encoder_new)
         priv->encoder = agent_class->encoder_new(agent);
+    if (priv->tag == 0) {
+        g_static_mutex_lock(&auto_tag_mutex);
+        priv->tag = auto_tag++;
+        if (priv->tag == 0)
+            priv->tag = auto_tag++;
+        g_static_mutex_unlock(&auto_tag_mutex);
+    }
+    apply_tag(priv);
 
     return object;
 }
@@ -143,6 +176,7 @@ milter_agent_init (MilterAgent *agent)
     priv->decoder = NULL;
     priv->writer = NULL;
     priv->reader = NULL;
+    priv->tag = 0;
     priv->finished = FALSE;
 }
 
@@ -186,10 +220,13 @@ set_property (GObject      *object,
 
     priv = MILTER_AGENT_GET_PRIVATE(object);
     switch (prop_id) {
-      case PROP_READER:
+    case PROP_READER:
         milter_agent_set_reader(MILTER_AGENT(object), g_value_get_object(value));
         break;
-      default:
+    case PROP_TAG:
+        milter_agent_set_tag(MILTER_AGENT(object), g_value_get_uint(value));
+        break;
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -211,6 +248,9 @@ get_property (GObject    *object,
     case PROP_DECODER:
         g_value_set_object(value, priv->decoder);
         break;
+    case PROP_TAG:
+        g_value_set_uint(value, priv->tag);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -253,7 +293,8 @@ cb_reader_flow (MilterReader *reader,
                                               MILTER_AGENT_ERROR_DECODE_ERROR,
                                               decoder_error,
                                               "Decode error");
-        milter_error("[agent][error][decode] %s", error->message);
+        milter_error("[%u] [agent][error][decode] %s",
+                     priv->tag, error->message);
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(user_data),
                                     error);
         g_error_free(error);
@@ -266,13 +307,16 @@ cb_reader_error (MilterReader *reader,
                  gpointer user_data)
 {
     GError *error = NULL;
+    MilterAgent *agent = user_data;
+    MilterAgentPrivate *priv;
 
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
     milter_utils_set_error_with_sub_error(&error,
                                           MILTER_AGENT_ERROR,
                                           MILTER_AGENT_ERROR_IO_ERROR,
                                           g_error_copy(reader_error),
                                           "Input error");
-    milter_error("[agent][error][reader] %s", error->message);
+    milter_error("[%u] [agent][error][reader] %s", priv->tag, error->message);
     milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(user_data),
                                 error);
     g_error_free(error);
@@ -298,6 +342,8 @@ milter_agent_set_reader (MilterAgent *agent, MilterReader *reader)
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     if (priv->reader) {
+        milter_reader_set_tag(priv->reader, 0);
+
 #define DISCONNECT(name)                                                \
         g_signal_handlers_disconnect_by_func(priv->reader,              \
                                              G_CALLBACK(cb_reader_ ## name), \
@@ -322,6 +368,8 @@ milter_agent_set_reader (MilterAgent *agent, MilterReader *reader)
         CONNECT(error);
         CONNECT(finished);
 #undef CONNECT
+
+        milter_reader_set_tag(priv->reader, priv->tag);
     }
 }
 
@@ -337,13 +385,17 @@ cb_writer_error (MilterWriter *writer,
                  gpointer user_data)
 {
     GError *error = NULL;
+    MilterAgent *agent = user_data;
+    MilterAgentPrivate *priv;
 
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
     milter_utils_set_error_with_sub_error(&error,
                                           MILTER_AGENT_ERROR,
                                           MILTER_AGENT_ERROR_IO_ERROR,
                                           g_error_copy(writer_error),
                                           "Output error");
-    milter_error("[agent][error][writer] %s", error->message);
+    milter_error("[%u] [agent][error][writer] %s",
+                 priv->tag, error->message);
     milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(user_data),
                                 error);
     g_error_free(error);
@@ -400,6 +452,8 @@ milter_agent_set_writer (MilterAgent *agent, MilterWriter *writer)
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
     if (priv->writer) {
+        milter_writer_set_tag(priv->writer, 0);
+
 #define DISCONNECT(name)                                                \
         g_signal_handlers_disconnect_by_func(priv->writer,              \
                                              G_CALLBACK(cb_writer_ ## name), \
@@ -422,6 +476,8 @@ milter_agent_set_writer (MilterAgent *agent, MilterWriter *writer)
         CONNECT(error);
         CONNECT(finished);
 #undef CONNECT
+
+        milter_writer_set_tag(priv->writer, priv->tag);
     }
 }
 
@@ -447,7 +503,7 @@ milter_agent_start (MilterAgent *agent)
     if (priv->reader)
         milter_reader_start(priv->reader);
     if (priv->writer)
-        milter_reader_start(priv->reader);
+        milter_writer_start(priv->writer);
 }
 
 void
@@ -457,7 +513,9 @@ milter_agent_shutdown (MilterAgent *agent)
 
     priv = MILTER_AGENT_GET_PRIVATE(agent);
 
+    milter_debug("[%u] [agent][shutdown]", priv->tag);
     if (priv->reader) {
+        milter_debug("[%u] [agent][shutdown][reader]", priv->tag);
         milter_reader_shutdown(priv->reader);
     } else {
         if (!priv->finished)
@@ -465,6 +523,22 @@ milter_agent_shutdown (MilterAgent *agent)
     }
 }
 
+guint
+milter_agent_get_tag (MilterAgent *agent)
+{
+    return MILTER_AGENT_GET_PRIVATE(agent)->tag;
+}
+
+void
+milter_agent_set_tag (MilterAgent *agent, guint tag)
+{
+    MilterAgentPrivate *priv;
+
+    priv = MILTER_AGENT_GET_PRIVATE(agent);
+    priv->tag = tag;
+    apply_tag(priv);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
