@@ -24,6 +24,7 @@
 #include "milter-utils.h"
 #include "milter-connection.h"
 #include "milter-esmtp.h"
+#include "milter-message-result.h"
 
 /* enumerations from "milter-protocol.h" */
 GType
@@ -387,6 +388,47 @@ milter_esmtp_error_get_type (void)
   }
   return etype;
 }
+GType
+milter_state_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_STATE_INVALID, "MILTER_STATE_INVALID", "invalid" },
+      { MILTER_STATE_START, "MILTER_STATE_START", "start" },
+      { MILTER_STATE_NEGOTIATE, "MILTER_STATE_NEGOTIATE", "negotiate" },
+      { MILTER_STATE_NEGOTIATE_REPLIED, "MILTER_STATE_NEGOTIATE_REPLIED", "negotiate-replied" },
+      { MILTER_STATE_CONNECT, "MILTER_STATE_CONNECT", "connect" },
+      { MILTER_STATE_CONNECT_REPLIED, "MILTER_STATE_CONNECT_REPLIED", "connect-replied" },
+      { MILTER_STATE_HELO, "MILTER_STATE_HELO", "helo" },
+      { MILTER_STATE_HELO_REPLIED, "MILTER_STATE_HELO_REPLIED", "helo-replied" },
+      { MILTER_STATE_ENVELOPE_FROM, "MILTER_STATE_ENVELOPE_FROM", "envelope-from" },
+      { MILTER_STATE_ENVELOPE_FROM_REPLIED, "MILTER_STATE_ENVELOPE_FROM_REPLIED", "envelope-from-replied" },
+      { MILTER_STATE_ENVELOPE_RECIPIENT, "MILTER_STATE_ENVELOPE_RECIPIENT", "envelope-recipient" },
+      { MILTER_STATE_ENVELOPE_RECIPIENT_REPLIED, "MILTER_STATE_ENVELOPE_RECIPIENT_REPLIED", "envelope-recipient-replied" },
+      { MILTER_STATE_DATA, "MILTER_STATE_DATA", "data" },
+      { MILTER_STATE_DATA_REPLIED, "MILTER_STATE_DATA_REPLIED", "data-replied" },
+      { MILTER_STATE_UNKNOWN, "MILTER_STATE_UNKNOWN", "unknown" },
+      { MILTER_STATE_UNKNOWN_REPLIED, "MILTER_STATE_UNKNOWN_REPLIED", "unknown-replied" },
+      { MILTER_STATE_HEADER, "MILTER_STATE_HEADER", "header" },
+      { MILTER_STATE_HEADER_REPLIED, "MILTER_STATE_HEADER_REPLIED", "header-replied" },
+      { MILTER_STATE_END_OF_HEADER, "MILTER_STATE_END_OF_HEADER", "end-of-header" },
+      { MILTER_STATE_END_OF_HEADER_REPLIED, "MILTER_STATE_END_OF_HEADER_REPLIED", "end-of-header-replied" },
+      { MILTER_STATE_BODY, "MILTER_STATE_BODY", "body" },
+      { MILTER_STATE_BODY_REPLIED, "MILTER_STATE_BODY_REPLIED", "body-replied" },
+      { MILTER_STATE_END_OF_MESSAGE, "MILTER_STATE_END_OF_MESSAGE", "end-of-message" },
+      { MILTER_STATE_END_OF_MESSAGE_REPLIED, "MILTER_STATE_END_OF_MESSAGE_REPLIED", "end-of-message-replied" },
+      { MILTER_STATE_QUIT, "MILTER_STATE_QUIT", "quit" },
+      { MILTER_STATE_QUIT_REPLIED, "MILTER_STATE_QUIT_REPLIED", "quit-replied" },
+      { MILTER_STATE_ABORT, "MILTER_STATE_ABORT", "abort" },
+      { MILTER_STATE_ABORT_REPLIED, "MILTER_STATE_ABORT_REPLIED", "abort-replied" },
+      { MILTER_STATE_FINISHED, "MILTER_STATE_FINISHED", "finished" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterState", values);
+  }
+  return etype;
+}
 
 /* Generated data ends here */
 
