@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -56,6 +56,7 @@ void data_feed_end_of_message (void);
 void test_feed_end_of_message (gconstpointer data);
 void test_feed_quit (void);
 void test_feed_abort (void);
+void test_feed_abort_outsize_message_processing (void);
 void test_write_error (void);
 void test_timeout (void);
 
@@ -127,9 +128,16 @@ static gsize body_chunk_size;
 static gchar *body_chunk_test_data;
 static gsize body_chunk_test_data_size;
 
+static gchar *end_of_message_chunk;
+static gsize end_of_message_chunk_size;
+static gchar *end_of_message_chunk_test_data;
+static gsize end_of_message_chunk_test_data_size;
+
 static gchar *unknown_command;
 static gsize unknown_command_size;
 
+static MilterClientContextState abort_state;
+
 static GList *timeout_ids;
 
 static GError *actual_error;
@@ -396,10 +404,17 @@ cb_body_response (MilterClientContext *context,
 }
 
 static MilterStatus
-cb_end_of_message (MilterClientContext *context, gpointer user_data)
+cb_end_of_message (MilterClientContext *context,
+                   const gchar *chunk, gsize size,
+                   gpointer user_data)
 {
     n_end_of_messages++;
 
+    if (end_of_message_chunk)
+        g_free(end_of_message_chunk);
+    end_of_message_chunk = g_strndup(chunk, size);
+    end_of_message_chunk_size = size;
+
     retrieve_context_info(context);
 
     milter_assert_equal_state(END_OF_MESSAGE);
@@ -415,9 +430,11 @@ cb_end_of_message_response (MilterClientContext *context,
 }
 
 static MilterStatus
-cb_abort (MilterClientContext *context, gpointer user_data)
+cb_abort (MilterClientContext *context, MilterClientContextState state,
+          gpointer user_data)
 {
     n_aborts++;
+    abort_state = state;
 
     retrieve_context_info(context);
 
@@ -576,9 +593,16 @@ setup (void)
     body_chunk_test_data = NULL;
     body_chunk_test_data_size = 0;
 
+    end_of_message_chunk = NULL;
+    end_of_message_chunk_size = 0;
+    end_of_message_chunk_test_data = NULL;
+    end_of_message_chunk_test_data_size = 0;
+
     unknown_command = NULL;
     unknown_command_size = 0;
 
+    abort_state = MILTER_CLIENT_CONTEXT_STATE_INVALID;
+
     timeout_ids = NULL;
 
     actual_error = NULL;
@@ -698,6 +722,11 @@ teardown (void)
     if (body_chunk_test_data)
         g_free(body_chunk_test_data);
 
+    if (end_of_message_chunk)
+        g_free(end_of_message_chunk);
+    if (end_of_message_chunk_test_data)
+        g_free(end_of_message_chunk_test_data);
+
     if (unknown_command)
         g_free(unknown_command);
 
@@ -942,8 +971,8 @@ test_feed_connect_with_macro (void)
     clear_reuse_data();
 
     expected_macros =
-        gcut_hash_table_string_string_new("j", "debian.cozmixng.org",
-                                          "daemon_name", "debian.cozmixng.org",
+        gcut_hash_table_string_string_new("j", "debian.example.com",
+                                          "daemon_name", "debian.example.com",
                                           "v", "Postfix 2.5.5",
                                           NULL);
     milter_command_encoder_encode_define_macro(encoder,
@@ -1042,12 +1071,12 @@ feed_envelope_from_pre_hook_with_macro (void)
     GHashTable *macros;
 
     macro_name = g_strdup("mail_addr");
-    expected_macro_value = g_strdup("kou@cozmixng.org");
+    expected_macro_value = g_strdup("kou@example.com");
     macros =
-        gcut_hash_table_string_string_new("{mail_addr}", "kou@cozmixng.org",
+        gcut_hash_table_string_string_new("{mail_addr}", "kou@example.com",
                                           NULL);
     expected_macros =
-        gcut_hash_table_string_string_new("mail_addr", "kou@cozmixng.org",
+        gcut_hash_table_string_string_new("mail_addr", "kou@example.com",
                                           NULL);
     milter_command_encoder_encode_define_macro(encoder,
                                        &packet, &packet_size,
@@ -1069,7 +1098,7 @@ void
 test_feed_envelope_from (gconstpointer data)
 {
     const HookFunction pre_hook = data;
-    const gchar from[] = "<kou@cozmixng.org>";
+    const gchar from[] = "<kou@example.com>";
 
     test_feed_helo(feed_helo_pre_hook_with_macro);
     clear_reuse_data();
@@ -1102,12 +1131,12 @@ feed_envelope_recipient_pre_hook_with_macro (void)
     GHashTable *macros;
 
     macro_name = g_strdup("{rcpt_addr}");
-    expected_macro_value = g_strdup("kou@cozmixng.org");
+    expected_macro_value = g_strdup("kou@example.com");
     macros =
-        gcut_hash_table_string_string_new("{rcpt_addr}", "kou@cozmixng.org",
+        gcut_hash_table_string_string_new("{rcpt_addr}", "kou@example.com",
                                           NULL);
     expected_macros =
-        gcut_hash_table_string_string_new("rcpt_addr", "kou@cozmixng.org",
+        gcut_hash_table_string_string_new("rcpt_addr", "kou@example.com",
                                           NULL);
     milter_command_encoder_encode_define_macro(encoder,
                                                &packet, &packet_size,
@@ -1131,7 +1160,7 @@ void
 test_feed_envelope_recipient (gconstpointer data)
 {
     const HookFunction pre_hook = data;
-    const gchar to[] = "<kou@cozmixng.org>";
+    const gchar to[] = "<kou@example.com>";
 
     test_feed_envelope_from(feed_envelope_from_pre_hook_with_macro);
     clear_reuse_data();
@@ -1400,13 +1429,13 @@ test_feed_body (gconstpointer data)
 }
 
 static void
-feed_end_of_message_pre_hook_with_body (void)
+feed_end_of_message_pre_hook_with_chunk (void)
 {
-    body_chunk_test_data = g_strdup("La de da de da 1.\n"
-                                    "La de da de da 2.\n"
-                                    "La de da de da 3.\n"
-                                    "La de da de da 4.");
-    body_chunk_test_data_size = strlen(body_chunk_test_data);
+    end_of_message_chunk_test_data = g_strdup("La de da de da 1.\n"
+                                              "La de da de da 2.\n"
+                                              "La de da de da 3.\n"
+                                              "La de da de da 4.");
+    end_of_message_chunk_test_data_size = strlen(end_of_message_chunk_test_data);
 }
 
 static void
@@ -1425,23 +1454,23 @@ feed_end_of_message_pre_hook_with_macro (void)
 }
 
 static void
-feed_end_of_message_pre_hook_with_body_and_macro (void)
+feed_end_of_message_pre_hook_with_chunk_and_macro (void)
 {
-    feed_end_of_message_pre_hook_with_body();
+    feed_end_of_message_pre_hook_with_chunk();
     feed_end_of_message_pre_hook_with_macro();
 }
 
 void
 data_feed_end_of_message (void)
 {
-    cut_add_data("without body",
+    cut_add_data("without chunk",
                  NULL, NULL,
-                 "without body - with macro",
+                 "without chunk - with macro",
                  feed_end_of_message_pre_hook_with_macro, NULL,
-                 "with body",
-                 feed_end_of_message_pre_hook_with_body, NULL,
-                 "with body - with macro",
-                 feed_end_of_message_pre_hook_with_body_and_macro, NULL);
+                 "with chunk",
+                 feed_end_of_message_pre_hook_with_chunk, NULL,
+                 "with chunk - with macro",
+                 feed_end_of_message_pre_hook_with_chunk_and_macro, NULL);
 }
 
 void
@@ -1456,20 +1485,22 @@ test_feed_end_of_message (gconstpointer data)
         pre_hook();
 
     milter_command_encoder_encode_end_of_message(encoder, &packet, &packet_size,
-                                                 body_chunk_test_data,
-                                                 body_chunk_test_data_size);
+                                                 end_of_message_chunk_test_data,
+                                                 end_of_message_chunk_test_data_size);
     gcut_assert_error(feed());
-    if (body_chunk_test_data) {
-        cut_assert_equal_int(2, n_bodies);
-        cut_assert_equal_int(2, n_body_responses);
-        cut_assert_equal_string(body_chunk_test_data, body_chunk);
-        cut_assert_equal_int(body_chunk_test_data_size, body_chunk_size);
-    } else {
-        cut_assert_equal_int(1, n_bodies);
-        cut_assert_equal_int(1, n_body_responses);
-    }
+
+    cut_assert_equal_int(1, n_bodies);
+    cut_assert_equal_int(1, n_body_responses);
     cut_assert_equal_int(1, n_end_of_messages);
     cut_assert_equal_int(1, n_end_of_message_responses);
+
+    if (end_of_message_chunk_test_data) {
+        cut_assert_equal_string(end_of_message_chunk_test_data,
+                                end_of_message_chunk);
+        cut_assert_equal_int(end_of_message_chunk_test_data_size,
+                             end_of_message_chunk_size);
+    }
+
     milter_assert_equal_state(END_OF_MESSAGE_REPLIED);
 
     gcut_assert_equal_hash_table_string_string(expected_macros, defined_macros);
@@ -1511,6 +1542,32 @@ test_feed_abort (void)
     cut_assert_equal_int(1, n_abort_responses);
     milter_assert_equal_state(ABORT_REPLIED);
 
+    gcut_assert_equal_enum(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                           MILTER_CLIENT_CONTEXT_STATE_BODY_REPLIED,
+                           abort_state);
+
+    milter_utils_merge_hash_string_string(expected_available_macros,
+                                          expected_macros);
+    gcut_assert_equal_hash_table_string_string(expected_available_macros,
+                                               available_macros);
+}
+
+void
+test_feed_abort_outsize_message_processing (void)
+{
+    test_feed_helo(feed_helo_pre_hook_with_macro);
+    clear_reuse_data();
+
+    milter_command_encoder_encode_abort(encoder, &packet, &packet_size);
+    gcut_assert_error(feed());
+    cut_assert_equal_int(1, n_aborts);
+    cut_assert_equal_int(1, n_abort_responses);
+    milter_assert_equal_state(ABORT_REPLIED);
+
+    gcut_assert_equal_enum(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                           MILTER_CLIENT_CONTEXT_STATE_HELO_REPLIED,
+                           abort_state);
+
     milter_utils_merge_hash_string_string(expected_available_macros,
                                           expected_macros);
     gcut_assert_equal_hash_table_string_string(expected_available_macros,
