@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -66,7 +66,7 @@ struct _MilterManagerTestClientPrivate
     gchar *envelope_recipient;
     gchar *header_name;
     gchar *header_value;
-    gchar *body_chunk;
+    GString *body_chunk;
     gchar *end_of_message_chunk;
     gchar *unknown_command;
 
@@ -115,7 +115,7 @@ milter_manager_test_client_init (MilterManagerTestClient *test_client)
     priv->envelope_recipient = NULL;
     priv->header_name = NULL;
     priv->header_value = NULL;
-    priv->body_chunk = NULL;
+    priv->body_chunk = g_string_new(NULL);
     priv->end_of_message_chunk = NULL;
     priv->unknown_command = NULL;
 
@@ -295,9 +295,10 @@ static void
 parse_body_info (MilterManagerTestClientPrivate *priv,
                  const gchar *chunk, gsize size)
 {
-    if (priv->body_chunk)
-        g_free(priv->body_chunk);
-    priv->body_chunk = receive_additional_info(chunk, size, "receive: body: ");
+    if (!priv->body_chunk)
+        priv->body_chunk = g_string_new(NULL);
+    g_string_append(priv->body_chunk,
+                    receive_additional_info(chunk, size, "receive: body: "));
 }
 
 static void
@@ -395,6 +396,7 @@ cb_output_received (GCutEgg *egg, const gchar *chunk, gsize size,
     g_string_append_len(priv->output_received, chunk, size);
     output_channel = gcut_egg_get_output(egg);
     if (chunk[size - 1] != '\n' ||
+        size == 4096 /* FIXME: 4096 is GCutEgg's buffer size */ ||
         g_io_channel_get_buffer_condition(output_channel) == G_IO_IN) {
         return;
     }
@@ -599,7 +601,7 @@ milter_manager_test_client_clear_data (MilterManagerTestClient *client)
     }
 
     if (priv->body_chunk) {
-        g_free(priv->body_chunk);
+        g_string_free(priv->body_chunk, TRUE);
         priv->body_chunk = NULL;
     }
 
@@ -731,7 +733,7 @@ milter_manager_test_client_set_arguments (MilterManagerTestClient *client,
     g_array_append_val(priv->command, test_client_path);
     append_command(priv->command, "--print-status");
     append_command(priv->command, "--timeout");
-    append_command(priv->command, "0.5");
+    append_command(priv->command, "1.5");
     append_command(priv->command, "--host");
     append_command(priv->command, "127.0.0.1");
     append_command(priv->command, "--port");
@@ -885,7 +887,12 @@ milter_manager_test_client_get_n_body_received (MilterManagerTestClient *client)
 const gchar *
 milter_manager_test_client_get_body_chunk (MilterManagerTestClient *client)
 {
-    return MILTER_MANAGER_TEST_CLIENT_GET_PRIVATE(client)->body_chunk;
+    GString *body_chunk;
+
+    body_chunk = MILTER_MANAGER_TEST_CLIENT_GET_PRIVATE(client)->body_chunk;
+    if (!body_chunk)
+        return NULL;
+    return body_chunk->len == 0 ? NULL : body_chunk->str;
 }
 
 guint
@@ -924,6 +931,12 @@ milter_manager_test_client_get_unknown_command (MilterManagerTestClient *client)
     return MILTER_MANAGER_TEST_CLIENT_GET_PRIVATE(client)->unknown_command;
 }
 
+gboolean
+milter_manager_test_client_is_reaped (MilterManagerTestClient *client)
+{
+    return MILTER_MANAGER_TEST_CLIENT_GET_PRIVATE(client)->reaped;
+}
+
 static gboolean
 cb_timeout_waiting (gpointer data)
 {
@@ -942,7 +955,7 @@ milter_manager_test_client_wait_reply (MilterManagerTestClient *client,
 
     timeout_waiting_id = g_timeout_add(100, cb_timeout_waiting,
                                        &timeout_waiting);
-    while (timeout_waiting && !getter(client))
+    while (timeout_waiting && getter(client) == 0)
         g_main_context_iteration(NULL, TRUE);
     g_source_remove(timeout_waiting_id);
 
@@ -952,27 +965,53 @@ milter_manager_test_client_wait_reply (MilterManagerTestClient *client,
 void
 milter_manager_test_clients_wait_reply (const GList *clients,
                                         MilterManagerTestClientGetNReceived getter)
+{
+    milter_manager_test_clients_wait_n_replies(clients, getter,
+                                               g_list_length((GList *)clients));
+}
+
+void
+milter_manager_test_clients_wait_n_replies(const GList *clients,
+                                           MilterManagerTestClientGetNReceived getter,
+                                           guint n_replies)
 {
     gboolean timeout_waiting = TRUE;
     guint timeout_waiting_id;
-    guint n_clients;
 
-    n_clients = g_list_length((GList *)clients);
-    timeout_waiting_id = g_timeout_add(100, cb_timeout_waiting,
+    timeout_waiting_id = g_timeout_add(1000, cb_timeout_waiting,
                                        &timeout_waiting);
     while (timeout_waiting &&
-           n_clients > milter_manager_test_clients_collect_n_received(clients,
+           n_replies > milter_manager_test_clients_collect_n_received(clients,
                                                                       getter)) {
         g_main_context_iteration(NULL, TRUE);
     }
     g_source_remove(timeout_waiting_id);
 
     cut_assert_true(timeout_waiting, "timeout");
-    cut_assert_equal_uint(n_clients,
+    cut_assert_equal_uint(n_replies,
                           milter_manager_test_clients_collect_n_received(clients,
                                                                          getter));
 }
 
+void
+milter_manager_test_clients_wait_n_alive(const GList *clients, guint n_alive)
+{
+    gboolean timeout_waiting = TRUE;
+    guint timeout_waiting_id;
+
+    timeout_waiting_id = g_timeout_add(1000, cb_timeout_waiting,
+                                       &timeout_waiting);
+    while (timeout_waiting &&
+           n_alive != milter_manager_test_clients_collect_n_alive(clients)) {
+        g_main_context_iteration(NULL, TRUE);
+    }
+    g_source_remove(timeout_waiting_id);
+
+    cut_assert_true(timeout_waiting, "timeout");
+    cut_assert_equal_uint(n_alive,
+                          milter_manager_test_clients_collect_n_alive(clients));
+}
+
 void
 milter_manager_test_client_assert_nothing_output (MilterManagerTestClient *client)
 {
@@ -1026,6 +1065,22 @@ milter_manager_test_clients_collect_n_received (const GList *clients,
     return n_received;
 }
 
+guint
+milter_manager_test_clients_collect_n_alive (const GList *clients)
+{
+    guint n_alive = 0;
+    const GList *node;
+
+    for (node = clients; node; node = g_list_next(node)) {
+        MilterManagerTestClient *client = node->data;
+
+        if (!milter_manager_test_client_is_reaped(client))
+            n_alive++;
+    }
+
+    return n_alive;
+}
+
 const GList *
 milter_manager_test_clients_collect_strings (const GList *clients,
                                              MilterManagerTestClientGetString getter)
@@ -1090,14 +1145,6 @@ milter_manager_test_clients_collect_negotiate_options (const GList *clients)
     return gcut_take_list(collected_options, safe_g_object_unref);
 }
 
-static void
-append_macro_data (gpointer key, gpointer value, gpointer user_data)
-{
-    GList **list = user_data;
-    *list = g_list_append(*list,
-                          milter_manager_test_pair_new(key, value));
-}
-
 const GList *
 milter_manager_test_clients_collect_macros (const GList *clients,
                                             MilterCommand command)
@@ -1109,7 +1156,8 @@ milter_manager_test_clients_collect_macros (const GList *clients,
         MilterManagerTestClient *client = node->data;
         GHashTable *macros;
         GHashTable *context_macro;
-        GList *list = NULL;
+        GList *macro_list = NULL;
+        GList *macro_names, *node;
 
         macros = (GHashTable*)milter_manager_test_client_get_macros(client);
         context_macro = g_hash_table_lookup(macros,
@@ -1117,9 +1165,18 @@ milter_manager_test_clients_collect_macros (const GList *clients,
         if (!context_macro)
             continue;
 
-        g_hash_table_foreach(context_macro,
-                             (GHFunc)append_macro_data, &list);
-        collected_macros = g_list_concat(collected_macros, list);
+        macro_names = g_hash_table_get_keys(context_macro);
+        macro_names = g_list_sort(macro_names, (GCompareFunc)g_utf8_collate);
+        for (node = macro_names; node; node = g_list_next(node)) {
+            const gchar *macro_name = node->data;
+            const gchar *macro_value;
+            MilterManagerTestPair *pair;
+
+            macro_value = g_hash_table_lookup(context_macro, macro_name);
+            pair = milter_manager_test_pair_new(macro_name, macro_value);
+            macro_list = g_list_append(macro_list, pair);
+        }
+        collected_macros = g_list_concat(collected_macros, macro_list);
     }
 
     return gcut_take_list(collected_macros,
