@@ -40,6 +40,7 @@ void test_user_name (void);
 void test_command (void);
 void test_command_options (void);
 void test_fallback_status (void);
+void test_evaluation_mode (void);
 void test_merge (void);
 void test_applicable_condition (void);
 void test_attach_applicable_conditions (void);
@@ -357,6 +358,30 @@ test_fallback_status (void)
                            milter_manager_egg_get_fallback_status(egg));
 }
 
+void
+test_evaluation_mode (void)
+{
+    const gchar spec[] = "inet:9999@127.0.0.1";
+    GError *error = NULL;
+
+    egg = milter_manager_egg_new("child-milter");
+    milter_manager_egg_set_connection_spec(egg, spec, &error);
+    gcut_assert_error(error);
+
+    cut_assert_false(milter_manager_egg_is_evaluation_mode(egg));
+    child = milter_manager_egg_hatch(egg);
+    cut_assert_not_null(child);
+    cut_assert_false(milter_manager_child_is_evaluation_mode(child));
+
+    milter_manager_egg_set_evaluation_mode(egg, TRUE);
+    cut_assert_true(milter_manager_egg_is_evaluation_mode(egg));
+
+    g_object_unref(child);
+    child = milter_manager_egg_hatch(egg);
+    cut_assert_not_null(child);
+    cut_assert_true(milter_manager_child_is_evaluation_mode(child));
+}
+
 void
 test_applicable_condition (void)
 {
@@ -486,6 +511,7 @@ test_to_xml (void)
                             "  <name>child-milter</name>\n"
                             "  <enabled>true</enabled>\n"
                             "  <fallback-status>accept</fallback-status>\n"
+                            "  <evaluation-mode>false</evaluation-mode>\n"
                             "</milter>\n",
                             actual_xml);
 
@@ -496,6 +522,7 @@ test_to_xml (void)
                             "  <name>child-milter</name>\n"
                             "  <enabled>true</enabled>\n"
                             "  <fallback-status>accept</fallback-status>\n"
+                            "  <evaluation-mode>false</evaluation-mode>\n"
                             "  <additional-field>VALUE</additional-field>\n"
                             "</milter>\n",
                             actual_xml);
