@@ -67,7 +67,7 @@ static gint n_unknown_responses;
 static MilterStatus response_status;
 
 void
-startup (void)
+cut_startup (void)
 {
     MILTER_TYPE_CLIENT_CONTEXT_ERROR;
     MILTER_TYPE_MANAGER_CHILDREN_ERROR;
@@ -308,7 +308,7 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (config)
         g_object_unref(config);
@@ -1111,6 +1111,7 @@ do_end_of_message_header (MilterManagerTestScenario *scenario,
     cut_assert(actual_headers);
     actual_header_list = milter_headers_get_list(actual_headers);
 
+    cut_set_message("[%s]", group);
     gcut_assert_equal_list(
         expected_headers, actual_header_list,
         (GEqualFunc)milter_header_equal,
@@ -1356,27 +1357,88 @@ data_scenario_basic (void)
 static void
 data_scenario_end_of_message_action (void)
 {
-    cut_add_data("quarantine", g_strdup("quarantine.txt"), g_free,
+    cut_add_data("quarantine",
+                 g_strdup("quarantine.txt"), g_free,
                  "quarantine - header-change",
                  g_strdup("quarantine-header-change.txt"), g_free,
-                 "add-header", g_strdup("add-header.txt"), g_free,
+                 "quarantine - evaluation",
+                 g_strdup("quarantine-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("add-header",
+                 g_strdup("add-header.txt"), g_free,
                  "add-header - with-leading-space",
                  g_strdup("add-header-with-leading-space.txt"), g_free,
-                 "insert-header", g_strdup("insert-header.txt"), g_free,
+                 "add-header - evaluation",
+                 g_strdup("add-header-evaluation.txt"), g_free,
+                 "add-header - evaluation - temporary failure",
+                 g_strdup("add-header-evaluation-temporary-failure.txt"), g_free,
+                 "add-header - evaluation - reject",
+                 g_strdup("add-header-evaluation-reject.txt"), g_free,
+                 "add-header - evaluation - discard",
+                 g_strdup("add-header-evaluation-discard.txt"), g_free,
+                 NULL);
+
+    cut_add_data("insert-header",
+                 g_strdup("insert-header.txt"), g_free,
                  "insert-header - with-leading-space",
                  g_strdup("insert-header-with-leading-space.txt"), g_free,
-                 "change-header", g_strdup("change-header.txt"), g_free,
+                 "insert-header - evaluation",
+                 g_strdup("insert-header-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("change-header",
+                 g_strdup("change-header.txt"), g_free,
                  "change-header - with-leading-space",
                  g_strdup("change-header-with-leading-space.txt"), g_free,
-                 "delete-header", g_strdup("delete-header.txt"), g_free,
-                 "change-from", g_strdup("change-from.txt"), g_free,
-                 "add-recipient", g_strdup("add-recipient.txt"), g_free,
-                 "delete-recipient", g_strdup("delete-recipient.txt"), g_free,
-                 "replace-body", g_strdup("replace-body.txt"), g_free,
-                 "progress", g_strdup("progress.txt"), g_free);
+                 "change-header - evaluation",
+                 g_strdup("change-header-evaluation.txt"), g_free,
+                 NULL);
 
-    cut_add_data("replace-body - overwrite",
-                 g_strdup("replace-body-overwrite.txt"), g_free);
+    cut_add_data("delete-header",
+                 g_strdup("delete-header.txt"), g_free,
+                 "delete-header - evaluation",
+                 g_strdup("delete-header-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("change-from",
+                 g_strdup("change-from.txt"), g_free,
+                 "change-from - evaluation",
+                 g_strdup("change-from-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("add-recipient",
+                 g_strdup("add-recipient.txt"), g_free,
+                 "add-recipient - evaluation",
+                 g_strdup("add-recipient-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("delete-recipient",
+                 g_strdup("delete-recipient.txt"), g_free,
+                 "delete-recipient - evaluation",
+                 g_strdup("delete-recipient-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("replace-body",
+                 g_strdup("replace-body.txt"), g_free,
+                 "replace-body - overwrite",
+                 g_strdup("replace-body-overwrite.txt"), g_free,
+                 "replace-body - evaluation",
+                 g_strdup("replace-body-evaluation.txt"), g_free,
+                 NULL);
+
+    cut_add_data("progress", g_strdup("progress.txt"), g_free,
+                 NULL);
+}
+
+static void
+data_scenario_helo (void)
+{
+    cut_add_data("helo - reject - evaluation",
+                 g_strdup("helo-reject-evaluation.txt"), g_free,
+                 "helo - temporary-failure - evaluation",
+                 g_strdup("helo-temporary-failure-evaluation.txt"), g_free,
+                 NULL);
 }
 
 static void
@@ -1388,6 +1450,8 @@ data_scenario_envelope_from (void)
                  g_strdup("envelope-from-reject.txt"), g_free,
                  "envelope-from - discard",
                  g_strdup("envelope-from-discard.txt"), g_free,
+                 "envelope-from - discard - evaluation",
+                 g_strdup("envelope-from-discard-evaluation.txt"), g_free,
                  "envelope-from - reject & discard",
                  g_strdup("envelope-from-reject-and-discard.txt"), g_free,
                  "envelope-from - reject & accept",
@@ -1536,6 +1600,7 @@ data_scenario (void)
 {
     data_scenario_basic();
     data_scenario_end_of_message_action();
+    data_scenario_helo();
     data_scenario_envelope_from();
     data_scenario_envelope_recipient();
     data_scenario_data();
