@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -32,7 +32,8 @@
 
 void test_negotiate_reply (void);
 void test_continue (void);
-void test_reply_code (void);
+void test_reply_code_temporary_failure (void);
+void test_reply_code_reject (void);
 void test_temporary_failure (void);
 void test_reject (void);
 void test_accept (void);
@@ -394,6 +395,23 @@ setup (void)
     context = milter_server_context_new();
     setup_signals(context);
 
+    {
+        MilterOption *option;
+        option = milter_option_new(6,
+                                   MILTER_ACTION_ADD_HEADERS |
+                                   MILTER_ACTION_CHANGE_BODY |
+                                   MILTER_ACTION_ADD_ENVELOPE_RECIPIENT |
+                                   MILTER_ACTION_DELETE_ENVELOPE_RECIPIENT |
+                                   MILTER_ACTION_CHANGE_HEADERS |
+                                   MILTER_ACTION_QUARANTINE |
+                                   MILTER_ACTION_CHANGE_ENVELOPE_FROM |
+                                   MILTER_ACTION_ADD_ENVELOPE_RECIPIENT_WITH_PARAMETERS |
+                                   MILTER_ACTION_SET_SYMBOL_LIST,
+                                   MILTER_STEP_SKIP);
+        milter_server_context_set_option(context, option);
+        g_object_unref(option);
+    }
+
     channel = gcut_string_io_channel_new(NULL);
     g_io_channel_set_encoding(channel, NULL, NULL);
     reader = milter_reader_io_channel_new(channel);
@@ -575,7 +593,9 @@ write_data (const gchar *data, gsize data_size)
 void
 test_negotiate_reply (void)
 {
-    option = milter_option_new(2, MILTER_ACTION_ADD_HEADERS, MILTER_STEP_NO_HEADERS);
+    option = milter_option_new(2,
+                               MILTER_ACTION_ADD_HEADERS,
+                               MILTER_STEP_NO_HEADERS);
     requests = milter_macros_requests_new();
     milter_macros_requests_set_symbols(requests,
                                        MILTER_COMMAND_HELO,
@@ -604,14 +624,33 @@ test_continue (void)
 }
 
 void
-test_reply_code (void)
+test_reply_code_temporary_failure (void)
+{
+    const gchar code[] = "451 4.7.1 Greylisting in action, please come back";
+
+    milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
+    write_data(packet, packet_size);
+    cut_assert_equal_int(1, n_reply_codes);
+    cut_assert_equal_uint(451, actual_reply_code);
+    cut_assert_equal_string("4.7.1", actual_reply_extended_code);
+    cut_assert_equal_string("Greylisting in action, please come back",
+                            actual_reply_message);
+    milter_assert_status(MILTER_STATUS_TEMPORARY_FAILURE);
+}
+
+void
+test_reply_code_reject (void)
 {
     const gchar code[] = "554 5.7.1 1% 2%% 3%%%";
+
     milter_reply_encoder_encode_reply_code(encoder, &packet, &packet_size, code);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_reply_codes);
     cut_assert_equal_uint(554, actual_reply_code);
-    milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    cut_assert_equal_string("5.7.1", actual_reply_extended_code);
+    cut_assert_equal_string("1% 2%% 3%%%",
+                            actual_reply_message);
+    milter_assert_status(MILTER_STATUS_REJECT);
 }
 
 void
@@ -800,7 +839,7 @@ test_quarantine (void)
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_quarantines);
     cut_assert_equal_string(reason, actual_quarantine_reason);
-    milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    milter_assert_status(MILTER_STATUS_QUARANTINE);
 }
 
 void
@@ -809,7 +848,7 @@ test_connection_failure (void)
     milter_reply_encoder_encode_connection_failure(encoder, &packet, &packet_size);
     write_data(packet, packet_size);
     cut_assert_equal_int(1, n_connection_failures);
-    milter_assert_status(MILTER_STATUS_NOT_CHANGE);
+    milter_assert_status(MILTER_STATUS_ERROR);
 }
 
 void
@@ -896,8 +935,18 @@ test_reading_timeout (void)
 void
 test_writing_timeout (void)
 {
+    GIOChannel *read_channel;
     gboolean timed_out_writing = FALSE;
 
+    read_channel = gcut_string_io_channel_new(NULL);
+    cut_take(read_channel, (CutDestroyFunction)g_io_channel_unref);
+    g_io_channel_set_encoding(read_channel, NULL, NULL);
+    gcut_string_io_channel_set_pipe_mode(read_channel, TRUE);
+    g_object_unref(reader);
+    reader = milter_reader_io_channel_new(read_channel);
+    milter_agent_set_reader(MILTER_AGENT(context), reader);
+    milter_agent_start(MILTER_AGENT(context));
+
     milter_server_context_set_writing_timeout(context, 1);
     g_signal_connect(context, "writing-timeout",
                      G_CALLBACK(cb_timeout), &timed_out_writing);
@@ -906,6 +955,7 @@ test_writing_timeout (void)
 
     g_io_channel_set_buffered(channel, FALSE);
     gcut_string_io_channel_set_buffer_limit(channel, 1);
+    gcut_string_io_channel_set_pipe_mode(channel, TRUE);
 
     milter_server_context_helo(context, "delian");
     while (!timed_out_writing && !timed_out_after)
