@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #include <milter/server.h>
 #include <milter/core.h>
 
-#define DEFAULT_NEGOTIATE_VERSION 8
+#define DEFAULT_NEGOTIATE_VERSION 6
 
 static gboolean verbose = FALSE;
 static gboolean output_message = FALSE;
@@ -49,8 +49,12 @@ static socklen_t connect_address_length = 0;
 static gchar *helo_host = NULL;
 static gchar *envelope_from = NULL;
 static gchar **recipients = NULL;
+static gint current_recipient = 0;
 static gchar **body_chunks = NULL;
 static gchar *unknown_command = NULL;
+static gchar *authenticated_name = NULL;
+static gchar *authenticated_type = NULL;
+static gchar *authenticated_author = NULL;
 static MilterHeaders *option_headers = NULL;
 
 #define PROGRAM_NAME "milter-test-server"
@@ -65,7 +69,7 @@ typedef enum
     MILTER_TEST_SERVER_ERROR_TIMEOUT
 } MilterTestServerError;
 
-typedef struct Message
+typedef struct _Message
 {
     MilterHeaders *headers;
     MilterHeaders *original_headers;
@@ -101,6 +105,21 @@ typedef struct _ProcessData
 #define WHITE_COLOR "\033[01;37m"
 #define NORMAL_COLOR "\033[00m"
 
+static void
+remove_recipient (GList **recipients, const gchar *recipient)
+{
+    GList *node;
+
+    for (node = *recipients; node; node = g_list_next(node)) {
+        const gchar *_recipient = node->data;
+
+        if (g_str_equal(_recipient, recipient)) {
+            *recipients = g_list_remove_link(*recipients, node);
+            return;
+        }
+    }
+}
+
 static void
 send_quit (MilterServerContext *context, ProcessData *data)
 {
@@ -133,8 +152,103 @@ set_macro (gpointer key, gpointer value, gpointer user_data)
 static void
 send_recipient (MilterServerContext *context)
 {
-    milter_server_context_envelope_recipient(context, *recipients);
-    recipients++;
+    milter_server_context_envelope_recipient(context,
+                                             *(recipients + current_recipient));
+    current_recipient++;
+}
+
+static void
+set_macros_for_connect (MilterProtocolAgent *agent)
+{
+    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_CONNECT,
+                                     "{daemon_name}", PROGRAM_NAME,
+                                     "{if_name}", "localhost",
+                                     "{if_addr}", "127.0.0.1",
+                                     "j", "milter-test-server",
+                                     NULL);
+}
+
+static void
+set_macros_for_helo (MilterProtocolAgent *agent)
+{
+    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_HELO,
+                                     "{tls_version}", "0",
+                                     "{cipher}", "0",
+                                     "{cipher_bits}", "0",
+                                     "{cert_subject}", "cert_subject",
+                                     "{cert_issuer}", "cert_issuer",
+                                     NULL);
+}
+
+static void
+set_macros_for_envelope_from (MilterProtocolAgent *agent)
+{
+    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
+                                     "i", "i",
+                                     "{mail_mailer}", "mail_mailer",
+                                     "{mail_host}", "mail_host",
+                                     "{mail_addr}", "mail_addr",
+                                     NULL);
+
+    if (authenticated_name)
+        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
+                                         "{auth_authen}", authenticated_name,
+                                         NULL);
+    if (authenticated_type)
+        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
+                                         "{auth_type}", authenticated_type,
+                                         NULL);
+    if (authenticated_author)
+        milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
+                                         "{auth_author}", authenticated_author,
+                                         NULL);
+}
+
+static void
+set_macros_for_envelope_recipient (MilterProtocolAgent *agent)
+{
+    gchar *recipients_address;
+
+    recipients_address = g_strjoinv(",", recipients);
+    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_RECIPIENT,
+                                     "{rcpt_mailer}", "rcpt_mailer",
+                                     "{rcpt_host}", "rcpt_host",
+                                     "{rcpt_addr}", recipients_address,
+                                     NULL);
+    g_free(recipients_address);
+}
+
+static void
+set_macros_for_data (MilterProtocolAgent *agent)
+{
+}
+
+static void
+set_macros_for_header (MilterProtocolAgent *agent)
+{
+}
+
+static void
+set_macros_for_end_of_header (MilterProtocolAgent *agent)
+{
+}
+
+static void
+set_macros_for_body (MilterProtocolAgent *agent)
+{
+}
+
+static void
+set_macros_for_end_of_message (MilterProtocolAgent *agent)
+{
+    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_END_OF_MESSAGE,
+                                     "{msg-id}", "msg-id",
+                                     NULL);
+}
+
+static void
+set_macros_for_unknown (MilterProtocolAgent *agent)
+{
 }
 
 static void
@@ -142,12 +256,16 @@ cb_continue (MilterServerContext *context, gpointer user_data)
 {
     ProcessData *data = user_data;
     MilterStepFlags step = MILTER_STEP_NONE;
+    MilterProtocolAgent *agent;
+
+    agent = MILTER_PROTOCOL_AGENT(context);
 
     if (data->option)
         step = milter_option_get_step(data->option);
 
     switch (milter_server_context_get_state(context)) {
     case MILTER_SERVER_CONTEXT_STATE_NEGOTIATE:
+        set_macros_for_connect(agent);
         if (!(step & MILTER_STEP_NO_CONNECT)) {
             const gchar *host_name;
 
@@ -177,37 +295,47 @@ cb_continue (MilterServerContext *context, gpointer user_data)
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_CONNECT:
+        set_macros_for_helo(agent);
         if (!(step & MILTER_STEP_NO_HELO)) {
             milter_server_context_helo(context, helo_host);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_HELO:
+        milter_server_context_reset_message_related_data(context);
+        set_macros_for_envelope_from(agent);
         if (!(step & MILTER_STEP_NO_ENVELOPE_FROM)) {
             milter_server_context_envelope_from(context, envelope_from);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM:
+        set_macros_for_envelope_recipient(agent);
         if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT)) {
             send_recipient(context);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
         if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT) &&
-            *recipients) {
+            *(recipients + current_recipient)) {
+            set_macros_for_envelope_recipient(agent);
             send_recipient(context);
             break;
         }
-        if (!(step & MILTER_STEP_NO_UNKNOWN) && unknown_command) {
+        if (!(step & MILTER_STEP_NO_UNKNOWN) &&
+            unknown_command &&
+            milter_option_get_version(data->option) >= 3) {
+            set_macros_for_unknown(agent);
             milter_server_context_unknown(context, unknown_command);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_UNKNOWN:
+        set_macros_for_data(agent);
         if (!(step & MILTER_STEP_NO_DATA) &&
-            milter_option_get_version(data->option) > 2) {
+            milter_option_get_version(data->option) >= 4) {
             milter_server_context_data(context);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_DATA:
+        set_macros_for_header(agent);
         if (!(step & MILTER_STEP_NO_HEADERS)) {
             MilterHeader *header;
             header = milter_headers_get_nth_header(option_headers, 1);
@@ -219,16 +347,20 @@ cb_continue (MilterServerContext *context, gpointer user_data)
         if (!(step & MILTER_STEP_NO_HEADERS) &&
             milter_headers_length(option_headers) > 0) {
             MilterHeader *header;
+
+            set_macros_for_header(agent);
             header = milter_headers_get_nth_header(option_headers, 1);
             milter_server_context_header(context, header->name, header->value);
             milter_headers_remove(option_headers, header);
             break;
         }
+        set_macros_for_end_of_header(agent);
         if (!(step & MILTER_STEP_NO_END_OF_HEADER)) {
             milter_server_context_end_of_header(context);
             break;
         }
     case MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER:
+        set_macros_for_body(agent);
         if (!(step & MILTER_STEP_NO_BODY)) {
             milter_server_context_body(context, *body_chunks, strlen(*body_chunks));
             body_chunks++;
@@ -237,10 +369,12 @@ cb_continue (MilterServerContext *context, gpointer user_data)
     case MILTER_SERVER_CONTEXT_STATE_BODY:
         if (!(step & MILTER_STEP_NO_BODY) &&
             *body_chunks) {
+            set_macros_for_body(agent);
             milter_server_context_body(context, *body_chunks, strlen(*body_chunks));
             body_chunks++;
             break;
         }
+        set_macros_for_end_of_message(agent);
         milter_server_context_end_of_message(context, NULL, 0);
         break;
     case MILTER_SERVER_CONTEXT_STATE_END_OF_MESSAGE:
@@ -281,8 +415,12 @@ cb_temporary_failure (MilterServerContext *context, gpointer user_data)
 
     switch (state) {
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
-        cb_continue(context, user_data);
-        break;
+        remove_recipient(&(data->message->recipients),
+                         *(recipients + current_recipient - 1));
+        if (data->message->recipients) {
+            cb_continue(context, user_data);
+            break;
+        }
     default:
         send_abort(context, data);
         data->success = FALSE;
@@ -300,8 +438,12 @@ cb_reject (MilterServerContext *context, gpointer user_data)
 
     switch (state) {
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
-        cb_continue(context, user_data);
-        break;
+        remove_recipient(&(data->message->recipients),
+                         *(recipients + current_recipient - 1));
+        if (data->message->recipients) {
+            cb_continue(context, user_data);
+            break;
+        }
     default:
         send_abort(context, data);
         data->success = FALSE;
@@ -326,7 +468,10 @@ cb_reply_code (MilterServerContext *context,
         g_free(data->reply_message);
     data->reply_message = g_strdup(message);
 
-    cb_continue(context, user_data);
+    if (400 <= code && code < 500)
+        cb_temporary_failure(context, user_data);
+    else
+        cb_reject(context, user_data);
 }
 
 static void
@@ -432,9 +577,8 @@ cb_delete_recipient (MilterServerContext *context, const gchar *recipient,
     ProcessData *data = user_data;
     MilterHeader *header;
     gchar *old_value;
-    
-    data->message->recipients = g_list_remove(data->message->recipients,
-                                              recipient);
+
+    remove_recipient(&(data->message->recipients), recipient);
 
     header = milter_headers_lookup_by_name(data->message->headers, "To");
     if (!header)
@@ -484,12 +628,11 @@ cb_progress (MilterServerContext *context, gpointer user_data)
 
 static void
 cb_quarantine (MilterServerContext *context, const gchar *reason,
-              gpointer user_data)
+               gpointer user_data)
 {
     ProcessData *data = user_data;
 
     data->quarantine_reason = g_strdup(reason);
-    send_abort(context, data);
 }
 
 static void
@@ -1035,6 +1178,16 @@ static const GOptionEntry option_entries[] =
      "CHUNK"},
     {"unknown", 0, 0, G_OPTION_ARG_STRING, &unknown_command,
      N_("Use COMMAND for unknown SMTP command."), "COMMAND"},
+    {"authenticated-name", 0, 0, G_OPTION_ARG_STRING, &authenticated_name,
+     N_("Use NAME for authenticated SASL login name. "
+        "(This is used for {auth_authen} macro value.)"),
+     "NAME"},
+    {"authenticated-type", 0, 0, G_OPTION_ARG_STRING, &authenticated_type,
+     N_("Use TYPE for authenticated SASL login method. "
+        "(This is used for {auth_type} macro value.)"), "TYPE"},
+    {"authenticated-author", 0, 0, G_OPTION_ARG_STRING, &authenticated_author,
+     N_("Use AUTHOR for authenticated SASL sender. "
+        "(This is used for {auth_author} macro value.)"), "AUTHOR"},
     {"mail-file", 'm', 0, G_OPTION_ARG_CALLBACK, parse_mail_file_arg,
      N_("Use mail placed at PATH as mail content."), "PATH"},
     {"output-message", 0,
@@ -1141,9 +1294,9 @@ post_option_parse (GOptionContext *option_context,
     if (!helo_host)
         helo_host = g_strdup("delian");
     if (!envelope_from)
-        envelope_from = g_strdup("<kou+send@cozmixng.org>");
+        envelope_from = g_strdup("<kou+send@example.com>");
     if (!recipients)
-        recipients = g_strsplit("<kou+receive@cozmixng.org>", ",", -1);
+        recipients = g_strsplit("<kou+receive@example.com>", ",", -1);
     if (!milter_headers_lookup_by_name(option_headers, "From"))
         milter_headers_add_header(option_headers, "From", envelope_from);
     if (!milter_headers_lookup_by_name(option_headers, "To"))
@@ -1187,7 +1340,8 @@ print_header (const gchar *prefix, MilterHeader *header)
         return;
     }
 
-    g_printf("%s%s %s: %s%s\n", color, prefix, header->name, *value_lines, NORMAL_COLOR);
+    g_printf("%s%s %s: %s%s\n", color, prefix, header->name, *value_lines,
+             NORMAL_COLOR);
     value_lines++;
 
     while (*value_lines) {
@@ -1335,7 +1489,7 @@ print_status (MilterServerContext *context, ProcessData *data)
     const gchar *status_name;
 
     status = milter_server_context_get_status(context);
-    if (status == MILTER_STATUS_NOT_CHANGE) {
+    if (MILTER_STATUS_IS_PASS(status)) {
         status_name = "pass";
     } else {
         GEnumValue *value;
@@ -1365,61 +1519,17 @@ print_result (MilterServerContext *context, ProcessData *data)
     }
 
     if (output_message) {
-        g_print("\n");
+        g_printf("\n");
         print_message(data->message);
-        g_print("\n");
+        g_printf("\n");
     }
 
-    g_print("elapsed-time: %g seconds\n", g_timer_elapsed(data->timer, NULL));
-}
-
-static void
-setup_default_macros (MilterServerContext *context)
-{
-    MilterProtocolAgent *agent = MILTER_PROTOCOL_AGENT(context);
-    gchar *recipients_address;
-
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_CONNECT,
-                                     "{daemon_name}", PROGRAM_NAME,
-                                     "{if_name}", "localhost",
-                                     "{if_addr}", "127.0.0.1",
-                                     "j", "milter-test-server",
-                                     NULL);
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_HELO,
-                                     "{tls_version}", "0",
-                                     "{cipher}", "0",
-                                     "{cipher_bits}", "0",
-                                     "{cert_subject}", "cert_subject",
-                                     "{cert_issuer}", "cert_issuer",
-                                     NULL);
-
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_FROM,
-                                     "i", "i",
-                                     "{auth_type}", "auth_type",
-                                     "{auth_authen}", "auth_authen",
-                                     "{auto_ssf}", "auto_ssf",
-                                     "{auto_author}", "auto_author",
-                                     "{mail_mailer}", "mail_mailer",
-                                     "{mail_host}", "mail_host",
-                                     "{mail_addr}", "mail_addr",
-                                     NULL);
-
-    recipients_address = g_strjoinv(",", recipients);
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_ENVELOPE_RECIPIENT,
-                                     "{rcpt_mailer}", "rcpt_mailer",
-                                     "{rcpt_host}", "rcpt_host",
-                                     "{rcpt_addr}", recipients_address,
-                                     NULL);
-    g_free(recipients_address);
-    milter_protocol_agent_set_macros(agent, MILTER_COMMAND_END_OF_MESSAGE,
-                                     "{msg-id}", "msg-id",
-                                     NULL);
+    g_printf("elapsed-time: %g seconds\n", g_timer_elapsed(data->timer, NULL));
 }
 
 static void
 setup_context (MilterServerContext *context, ProcessData *process_data)
 {
-    setup_default_macros(context);
     milter_server_context_set_name(context, PROGRAM_NAME);
 
     g_signal_connect(context, "ready", G_CALLBACK(cb_ready), process_data);
@@ -1460,6 +1570,7 @@ main (int argc, char *argv[])
     ProcessData process_data;
 
     milter_init();
+    milter_server_init();
 
     option_context = g_option_context_new(NULL);
     g_option_context_add_main_entries(option_context, option_entries, NULL);
@@ -1488,6 +1599,7 @@ main (int argc, char *argv[])
     g_object_unref(context);
     free_process_data(&process_data);
 
+    milter_server_quit();
     milter_quit();
     g_option_context_free(option_context);
     free_option_values();
